'use strict';

var path = require('node:path');
var fs = require('node:fs');

function isObject(value) {
  return value !== null && typeof value === "object";
}
function _defu(baseObject, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObject, {}, namespace, merger);
  }
  const object = Object.assign({}, defaults);
  for (const key in baseObject) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const value = baseObject[key];
    if (value === null || value === void 0) {
      continue;
    }
    if (merger && merger(object, key, value, namespace)) {
      continue;
    }
    if (Array.isArray(value) && Array.isArray(object[key])) {
      object[key] = [...value, ...object[key]];
    } else if (isObject(value) && isObject(object[key])) {
      object[key] = _defu(
        value,
        object[key],
        (namespace ? `${namespace}.` : "") + key.toString(),
        merger
      );
    } else {
      object[key] = value;
    }
  }
  return object;
}
function createDefu(merger) {
  return (...arguments_) => (
    // eslint-disable-next-line unicorn/no-array-reduce
    arguments_.reduce((p, c) => _defu(p, c, "", merger), {})
  );
}
const defu = createDefu();

// import { color as d3Color } from 'd3-color'
function cmkdir(dir) {
    const existed = fs.existsSync(dir);
    if (!existed) {
        fs.mkdirSync(dir, {
            recursive: true
        });
    }
}
function getAbsPath(p) {
    return path.isAbsolute(p) ? p : path.resolve(process.cwd(), p);
}
function renderTemplete(src, replacement) {
    let t = fs.readFileSync(src, 'utf8');
    if (replacement) {
        const replacements = Object.entries(replacement);
        for (const [key, value] of replacements) {
            t = t.replace(key, value);
        }
    }
    return t;
}
function withOpacityValue(variable) {
    return ({ opacityValue }) => {
        if (opacityValue === undefined) {
            return `rgb(var(${variable}))`;
        }
        return `rgb(var(${variable}) / ${opacityValue})`;
    };
}
function getKey(str) {
    return str;
}
/**
 * @description remove css variables -- prefix
 */
function removePrefix(name) {
    if (name.length > 2) {
        return name.slice(2);
    }
    return name;
}
function getJsValue(str) {
    return `withOpacityValue('${str}')`;
}
function getScssValue(str) {
    return `rgb(var(${str}))`;
}
// export function parseColor (cssColorSpecifier:string) {
//   return d3Color(cssColorSpecifier)
// }

function getOption(option) {
    var _a, _b, _c, _d, _e, _f, _g;
    const filesDefault = {
        extendColors: {
            getVarName: removePrefix,
            getVarValue: getJsValue
        },
        variables: {
            getVarName: removePrefix,
            getVarValue: getScssValue
        },
        root: {
            replacement: {
            // filepath: '{{filepath}}',
            // variableName: '{{variableName}}'
            }
        },
        export: {
            replacement: {
            // filepath: '{{filepath}}',
            // variableName: '{{variableName}}'
            }
        },
        util: true
    };
    return {
        entryPoint: getAbsPath(option.entryPoint),
        outdir: (_a = option.outdir) !== null && _a !== void 0 ? _a : '.',
        files: defu(option.files, filesDefault),
        sassOptions: (_b = option.sassOptions) !== null && _b !== void 0 ? _b : {},
        outputFileSystem: (_c = option.outputFileSystem) !== null && _c !== void 0 ? _c : fs,
        write: Boolean((_d = option.write) !== null && _d !== void 0 ? _d : true),
        intelliSense: defu(option.intelliSense, {
            getVarName: removePrefix,
            getVarValue: getKey
        }),
        injectBase: (_e = option.injectBase) !== null && _e !== void 0 ? _e : true,
        injectSelector: (_f = option.injectSelector) !== null && _f !== void 0 ? _f : ':root',
        withOpacityValue: (_g = option.withOpacityValue) !== null && _g !== void 0 ? _g : true,
        generateResult: {
            js: {},
            scss: {},
            meta: [],
            mergedMap: {}
        }
    };
}

const MergedMapPlaceholder = ':root';

// The asynchronous variants are much slower
function exposeScssVariable(exposeFilePath, options) {
    let sass;
    try {
        sass = require('sass');
    }
    catch (error) {
        console.error('Please install sass ! run `npm i -D sass` or `yarn add -D sass` and try again.');
        throw error;
    }
    const exposeAarry = [];
    const defaultOption = {
        functions: {
            'expose($map)': (args) => {
                const value = args[0].assertMap('map');
                const map = value.contents;
                exposeAarry.push({
                    map
                });
                return value;
            },
            'expose-with-selector($map,$selector)': (args) => {
                const value = args[0].assertMap('map');
                const selector = args[1].assertString('selector');
                const map = value.contents;
                exposeAarry.push({
                    selector: selector.text,
                    map
                });
                return value;
            }
        }
    };
    sass.compile(exposeFilePath, defu(options, defaultOption));
    return exposeAarry;
}
function extractTheme(exposeAarry) {
    var _a;
    const mergedMap = {};
    for (const item of exposeAarry) {
        for (const [key, color] of item.map) {
            const selector = (_a = item.selector) !== null && _a !== void 0 ? _a : MergedMapPlaceholder;
            if (mergedMap[selector]) {
                mergedMap[selector][key.text] = `${color.red} ${color.green} ${color.blue}`; // ${color.alpha}
            }
            else {
                mergedMap[selector] = {
                    [key.text]: `${color.red} ${color.green} ${color.blue}`
                };
            }
        }
    }
    return mergedMap;
}

function generateSync(option) {
    const opt = getOption(option);
    const { entryPoint, outdir, files, write, outputFileSystem: fs, intelliSense } = opt;
    const mergedMap = extractTheme(exposeScssVariable(entryPoint));
    const targetDir = path.dirname(entryPoint);
    const absOutdir = path.resolve(targetDir, outdir);
    const microtaskProducer = [];
    const result = {
        js: {},
        scss: {},
        meta: [],
        mergedMap: {}
    };
    result.mergedMap = mergedMap;
    const keys = [
        ...Object.values(mergedMap).reduce((acc, cur) => {
            for (const k of Object.keys(cur)) {
                acc.add(k);
            }
            return acc;
        }, new Set())
    ];
    const { getVarName, getVarValue } = intelliSense;
    result.meta = keys.map((x) => {
        var _a, _b;
        return {
            name: (_a = getVarName === null || getVarName === void 0 ? void 0 : getVarName(x)) !== null && _a !== void 0 ? _a : x,
            value: (_b = getVarValue === null || getVarValue === void 0 ? void 0 : getVarValue(x)) !== null && _b !== void 0 ? _b : x
        };
    });
    write && cmkdir(absOutdir);
    console.info('[Output Dir]: ' + absOutdir);
    if (files.variables !== false) {
        const { getVarName, getVarValue, outfile } = files.variables;
        // scss
        // ${{ removeColorPrefix(k) }}:{{ scssFilterShadow(k) }};
        const scssResult = keys
            .map((x) => {
            return `$${getVarName(x)}:${getVarValue(x)};`;
        })
            .join('\n');
        result.scss.variables = scssResult;
        if (write) {
            microtaskProducer.push(() => {
                fs.writeFileSync(getAbsPath(outfile !== null && outfile !== void 0 ? outfile : path.resolve(absOutdir, 'variables.scss')), scssResult);
                console.log('[variables.scss] generate Successfully!');
            });
        }
    }
    if (files.extendColors !== false) {
        // js:
        // '{{ removeColorPrefix(k) }}':{{ jsFilterShadow(k) }},
        const filename = 'extendColors.js';
        const { getVarName, getVarValue, outfile } = files.extendColors;
        const jsResult = keys
            .map((x) => {
            return `'${getVarName(x)}': ${getVarValue(x)},`;
        })
            .join('\n    ');
        result.js.extendColors = jsResult;
        const extendColorsTemplete = renderTemplete(path.resolve(__dirname, `./t/js/${filename}`), {
            '/* {{placeholder}} */': jsResult
        });
        if (write) {
            microtaskProducer.push(() => {
                fs.writeFileSync(getAbsPath(outfile !== null && outfile !== void 0 ? outfile : path.resolve(absOutdir, filename)), extendColorsTemplete);
                console.log(`[${filename}] generate Successfully!`);
            });
        }
    }
    function handleScssFile(key, filename) {
        const file = files[key];
        if (file !== false) {
            const src = path.resolve(__dirname, `./t/scss/${filename}`);
            const dest = path.resolve(absOutdir, filename);
            if (file === true) {
                if (write) {
                    fs.copyFileSync(src, dest);
                }
            }
            else {
                const { replacement, outfile } = file;
                const content = renderTemplete(src, replacement);
                if (write) {
                    fs.writeFileSync(getAbsPath(outfile !== null && outfile !== void 0 ? outfile : dest), content, 'utf8');
                }
            }
            write && console.log(`[${filename}] generate Successfully!`);
        }
    }
    microtaskProducer.push(() => handleScssFile('util', 'util.scss'));
    microtaskProducer.push(() => handleScssFile('export', 'export.scss'));
    microtaskProducer.push(() => handleScssFile('root', 'root.scss'));
    microtaskProducer.map((fn) => fn());
    return result;
}
const generate = generateSync;

function createPreset(options) {
    const defaults = {
        files: {
            root: false,
            util: false,
            variables: false,
            export: false,
            extendColors: {
                getVarName: getKey,
                getVarValue: getKey
            }
        },
        write: false,
        withOpacityValue: true,
        injectBase: true
    };
    const { meta } = generateSync(defu(options, defaults));
    return {
        content: [],
        theme: {
            extend: {
                colors: meta.reduce((acc, cur) => {
                    acc[cur.name] = withOpacityValue(cur.value);
                    return acc;
                }, {})
            }
        }
    };
}

const withOptions = (pluginFunction, configFunction = () => ({ content: [] })) => {
    const optionsFunction = (options) => {
        const defaults = {
            files: {
                root: false,
                util: false,
                variables: false,
                export: false,
                extendColors: false
            },
            write: false,
            withOpacityValue: true,
            injectBase: true
        };
        const generateResult = generateSync(defu(options, defaults));
        options.generateResult = generateResult;
        return {
            __options: options,
            handler: pluginFunction(options),
            config: configFunction(options)
        };
    };
    optionsFunction.__isOptionsFunction = true;
    // Expose plugin dependencies so that `object-hash` returns a different
    // value if anything here changes, to ensure a rebuild is triggered.
    optionsFunction.__pluginFunction = pluginFunction;
    optionsFunction.__configFunction = configFunction;
    return optionsFunction;
};
const createPlugin = withOptions((options) => {
    const { generateResult, injectSelector } = options;
    const mergedMap = generateResult.mergedMap;
    const params = mergedMap;
    if (injectSelector && mergedMap[MergedMapPlaceholder]) {
        mergedMap[injectSelector] = mergedMap[MergedMapPlaceholder];
        delete mergedMap[MergedMapPlaceholder];
    }
    return ({ addBase }) => {
        addBase(params);
    };
}, (options) => {
    const { generateResult } = options;
    return {
        content: [],
        theme: {
            extend: {
                colors: generateResult.meta.reduce((acc, cur) => {
                    acc[cur.name] = withOpacityValue(cur.value);
                    return acc;
                }, {})
            }
        }
    };
});

exports.createPlugin = createPlugin;
exports.createPreset = createPreset;
exports.exposeScssVariable = exposeScssVariable;
exports.extractTheme = extractTheme;
exports.generate = generate;
exports.generateSync = generateSync;
